@@ -23,6 +23,7 @@
 const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gdk = imports.gi.Gdk;
+const Gio = imports.gi.Gio;
 const Gtk = imports.gi.Gtk;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
@@ -35,7 +36,6 @@ const Geoclue = imports.geoclue;
 const LayersPopover = imports.layersPopover;
 const LocationServiceNotification = imports.locationServiceNotification;
 const MapView = imports.mapView;
-const MapSource = imports.mapSource;
 const PlaceEntry = imports.placeEntry;
 const PlaceStore = imports.placeStore;
 const Sidebar = imports.sidebar;
@@ -78,10 +78,6 @@ const MainWindow = new Lang.Class({
 
         this._overlay.add(this._mapView);
 
-        if (!this.application.local_tile_path) {
-            this._overlay.add_overlay(new MapSource.AttributionLogo());
-        }
-
         this._mapView.gotoUserLocation(false);
 
         this._sidebar = this._createSidebar();
@@ -100,6 +96,7 @@ const MainWindow = new Lang.Class({
         this._initActions();
         this._initSignals();
         this._restoreWindowGeometry();
+        this._initDND();
 
         this._busySignalId = 0;
 
@@ -144,6 +141,28 @@ const MainWindow = new Lang.Class({
         return sidebar;
     },
 
+    _initDND: function() {
+        this.drag_dest_set(Gtk.DestDefaults.DROP, null, 0);
+        this.drag_dest_add_uri_targets();
+
+        this.connect('drag-motion', (function(widget, ctx, x, y, time) {
+            return true;
+        }).bind(this));
+
+        this.connect('drag-data-received', (function(widget, ctx, x, y, data, info, time) {
+            let uri = data.get_uris()[0];
+            let content_type = Gio.content_type_guess(uri, null)[0];
+
+            if (content_type === 'application/vnd.geo+json' ||
+                content_type === 'application/json') {
+                this._mapView.openGeoJSON(Gio.file_new_for_uri(uri));
+                Gtk.drag_finish(ctx, true, true, time);
+            } else {
+                Gtk.drag_finish(ctx, false, false, time);
+            }
+        }).bind(this));
+    },
+
     _initActions: function() {
         Utils.addActions(this, {
             'close': {
