@@ -29,9 +29,9 @@ const Lang = imports.lang;
 const Application = imports.application;
 const ContactPlace = imports.contactPlace;
 const Geoclue = imports.geoclue;
+const GeoJSONSource = imports.geoJSONSource;
 const Location = imports.location;
 const Maps = imports.gi.GnomeMaps;
-const MapSource = imports.mapSource;
 const MapWalker = imports.mapWalker;
 const Place = imports.place;
 const PlaceMarker = imports.placeMarker;
@@ -42,8 +42,10 @@ const Utils = imports.utils;
 
 const MapType = {
     LOCAL: 'MapsLocalSource',
-    STREET: 'MapsStreetSource',
-    AERIAL: 'MapsAerialSource'
+    STREET:  Champlain.MAP_SOURCE_OSM_MAPQUEST,
+    AERIAL:  Champlain.MAP_SOURCE_OSM_AERIAL_MAP,
+    CYCLING: Champlain.MAP_SOURCE_OSM_CYCLE_MAP,
+    TRANSIT: Champlain.MAP_SOURCE_OSM_TRANSPORT_MAP
 };
 
 const MapMinZoom = 2;
@@ -81,6 +83,7 @@ const MapView = new Lang.Class({
         this.view = this._initView();
         this._initLayers();
 
+        this._factory = Champlain.MapSourceFactory.dup_default();
         this.setMapType(mapType);
 
         this._updateUserLocation();
@@ -132,6 +135,9 @@ const MapView = new Lang.Class({
 
         this._instructionMarkerLayer = new Champlain.MarkerLayer({ selection_mode: mode });
         this.view.add_layer(this._instructionMarkerLayer);
+
+        this._annotationMarkerLayer = new Champlain.MarkerLayer({ selection_mode: mode });
+        this.view.add_layer(this._annotationMarkerLayer);
     },
 
     _connectRouteSignals: function() {
@@ -154,15 +160,8 @@ const MapView = new Lang.Class({
             return;
 
         if (mapType !== MapType.LOCAL) {
-            if (mapType === MapType.AERIAL)
-                this.view.map_source = MapSource.createAerialSource();
-            else
-                this.view.map_source = MapSource.createStreetSource();
-
-            if (!this._attribution) {
-                this._attribution = new MapSource.AttributionLogo(this.view);
-                this.view.add_child(this._attribution);
-            }
+            let source = this._factory.create_cached_source(mapType);
+            this.view.map_source = source;
         } else {
             let renderer = new Champlain.ImageRenderer();
             let source = new Maps.FileTileSource({
@@ -185,6 +184,42 @@ const MapView = new Lang.Class({
         }
     },
 
+    _openGeoJSONInternal: function(file) {
+        if (!this.view.realized)
+            return;
+
+        try {
+            this._annotationMarkerLayer.remove_all();
+            let geoJSONSource = new GeoJSONSource.GeoJSONSource({
+                file: file,
+                mapView: this,
+                markerLayer: this._annotationMarkerLayer
+            });
+            geoJSONSource.parse();
+
+            if (this._annotationSource)
+                this.view.remove_overlay_source(this._annotationSource);
+
+            this._annotationSource = geoJSONSource;
+            this.view.add_overlay_source(this._annotationSource, 255);
+            if (geoJSONSource.bbox.is_valid())
+                this._gotoBBox(geoJSONSource.bbox);
+        } catch(e) {
+            let msg = _("Failed to parse GeoJSON file");
+            Application.notificationManager.showMessage(msg);
+            Utils.debug("failed to parse geojson file: %s".format(e.message));
+        }
+    },
+
+    openGeoJSON: function(file) {
+        if (this.view.realized) {
+            this._openGeoJSONInternal(file);
+        } else {
+            this.view.connect('notify::realized',
+                              this._openGeoJSONInternal.bind(this, file));
+        }
+    },
+
     gotoUserLocation: function(animate) {
         if (!this._userLocation)
             return;
